#!/usr/bin/env python

from nidibot.server_provider.server_provider_interface import (
    ServerProviderInterface,
    ServerStatus,
)


class GameServer:
    """
    A wrapper class for easier management of single game server instance.
    """

    def __init__(
        self, server_provider: ServerProviderInterface, server_id: str, server_name: str
    ) -> None:
        if not server_id:
            raise ValueError("Server ID is required!")

        self.__server_provider = server_provider
        self.__server_id = server_id
        self.__server_name = server_name

    def name(self) -> str:
        """
        States server name.

        Returns:
            server_name (str)
        """
        return self.__server_name

    def status(self) -> ServerStatus:
        """
        Returns server name.

        Returns:
            server_status (ServerStatus): server status as a structure
        """
        return self.__server_provider.status(self.__server_id)

    def start(self) -> bool:
        """
        Requests server provider to start server.

        Returns:
            result (bool): operation result
        """
        return self.__server_provider.start(self.__server_id)

    def stop(self) -> bool:
        """
        Requests server provider to stop server.

        Returns:
            result (bool): operation result
        """
        return self.__server_provider.stop(self.__server_id)

    def restart(self) -> bool:
        """
        Requests server provider to restart server.

        Returns:
            result (bool): operation result
        """
        return self.__server_provider.restart(self.__server_id)

    def create_backup(self) -> bool:
        """
        Requests server provider to create a game server backup.

        Returns:
            result (bool): operation result
        """
        return self.__server_provider.create_backup(self.__server_id)

    def restore_backup(self) -> bool:
        """
        Requests server provider to restore a game server backup.

        Returns:
            result (bool): operation result
        """
        return self.__server_provider.restore_backup(self.__server_id)



#!/usr/bin/env python

from dataclasses import dataclass, field
from abc import ABC, abstractmethod


@dataclass
class ServerStatus:
    game_name: str = ""
    game_version: str = ""
    server_address: str = ""
    status: str = ""
    available_until: str = ""
    update_available: bool = False
    players_limit: int = 0
    players_connected: int = 0
    player_names: list = field(default_factory=list)


class ServerProviderInterface(ABC):
    @abstractmethod
    def _poll(self) -> None:
        pass

    @abstractmethod
    def name(self) -> str:
        pass

    @abstractmethod
    def get_servers(self) -> list:
        pass

    @abstractmethod
    def status(self, server_id: str = "") -> ServerStatus:
        pass

    @abstractmethod
    def start(self, server_id: str = "") -> bool:
        pass

    @abstractmethod
    def stop(self, server_id: str = "") -> bool:
        pass

    @abstractmethod
    def restart(self, server_id: str = "") -> bool:
        pass

    @abstractmethod
    def create_backup(self, server_id: str = "") -> bool:
        pass

    @abstractmethod
    def restore_backup(self, server_id: str = "") -> bool:
        pass
